
import pygame

class AnimatedSprite(object):
    def __init__(self, surface, frames_per_animation, number_of_animations):
        self.sprite_sheet = SpriteSheet(surface, frames_per_animation, number_of_animations, True)
        self.frames_per_animation = frames_per_animation
        self.animation_index = 0
        self.frame_index = 0
        self.do_mirror = False
        
    def draw(self, dest_surface, dest_rect):
        self.sprite_sheet.draw(dest_surface, dest_rect)
    
    def next_frame(self, animation_index, do_mirror):
        if (animation_index == self.animation_index) and (do_mirror == self.do_mirror):
            self.frame_index = (self.frame_index + 1) % self.frames_per_animation
        else:
            self.frame_index = 0
            self.animation_index = animation_index
            self.do_mirror = do_mirror
            
        self.sprite_sheet.set_sprite(self.frame_index, self.animation_index, do_mirror)
     
     
        
class SpriteSheet(object):
    def __init__(self, surface, column_count, row_count, allow_mirror = False):
        self.source_surface_original = surface
        
        if allow_mirror:
            self.source_surface_mirror = pygame.transform.flip(self.source_surface_original, True, False)
        
        self._init_dimensions(column_count, row_count)
        self.set_sprite(0, 0)
        
        
    def _init_dimensions(self, column_count, row_count):
        self.column_count = column_count
        self.row_count = row_count
        self.frame_width = self.source_surface_original.get_width() / column_count
        self.frame_height = self.source_surface_original.get_height() / row_count
        
        
    def set_sprite(self, col_index, row_index, do_mirror = False):
        if not do_mirror:
            x = col_index * self.frame_width
            self.source_surface = self.source_surface_original
        else:
            x = (self.column_count - 1 - col_index) * self.frame_width
            self.source_surface = self.source_surface_mirror
            
        y = row_index * self.frame_height
        self.source_rect = pygame.Rect(x, y, self.frame_width, self.frame_height)
    
    def draw(self, dest_surface, dest_rect):
        dest_surface.blit(self.source_surface, dest_rect, self.source_rect)
    
def load_image(filename, transparent_color):
    surface = pygame.image.load(filename)
    surface.set_colorkey(transparent_color, pygame.RLEACCEL)
    return surface
    