
import pygame
import sprite

ORD_A = ord('A')
ORD_Z = ord('Z')
ORD_a = ord('a')
ORD_z = ord('z')
ORD_0 = ord('0')
ORD_9 = ord('9')
ORD_PERIOD = ord('.')
ORD_COMMA = ord(',')
ORD_COLON = ord(':')
ORD_KEY_LEFT = ord('<')
ORD_KEY_RIGHT = ord('>')
ORD_KEY_ESC = ord('^')
ORD_KEY_SPCA = ord('[')
ORD_KEY_SPCB = ord(']')
ORD_MINUS = ord('-')

ALIGN_LEFT = 1
ALIGN_CENTER = 2
ALIGN_RIGHT = 3

CHAR_WIDTH = 24
CHAR_HEIGHT = 32
COLUMN_COUNT = 13
ROW_COUNT = 4

def load_font_from_file(filename):
    image = sprite.load_image(filename, 0xffffff)
    return PixelFont(image)

class PixelFont(object):
    def __init__(self, source_surface):
        self.sprite_sheet = sprite.SpriteSheet(source_surface, COLUMN_COUNT, ROW_COUNT)
        
    def draw_string(self, message, surface, x, y, alignment):
        char_rect = self._get_first_char_rect(message, x, y, alignment)
        for char in message:
            col, row = self._char_to_coordinates(char)
            self.sprite_sheet.set_sprite(col, row)
            self.sprite_sheet.draw(surface, char_rect)
            char_rect = self._get_next_char_rect(char_rect) #TODO: modify in place for higher performance?
            
            
    def _get_first_char_rect(self, message, x, y, alignment):
        string_rect = pygame.Rect(x, y, len(message)*CHAR_WIDTH, CHAR_HEIGHT)
        if alignment == ALIGN_CENTER:
            string_rect.centerx = x
        elif alignment == ALIGN_RIGHT:
            string_rect.right = x
        char_rect = pygame.Rect(string_rect.x, string_rect.y, CHAR_WIDTH, CHAR_HEIGHT)
        return char_rect
    
    def _get_next_char_rect(self, last_char_rect):
        return last_char_rect.move(CHAR_WIDTH, 0)
    
    def _char_to_coordinates(self, char):
        num = ord(char)
        if num >= ORD_A and num <= ORD_Z:
            num = num - ORD_A
            x = num % 13
            y = 1 if num >= 13 else 0
        elif ORD_a <= num and num <= ORD_z:
            num = num - ORD_a
            x = num % 13
            y = 1 if num >= 13 else 0
        elif ORD_0 <= num and num <= ORD_9:
            x = num - ORD_0
            y = 2
        elif num == ORD_COLON:
            x = 0
            y = 3
        elif num == ORD_PERIOD:
            x = 1
            y = 3
        elif num == ORD_COMMA:
            x = 2
            y = 3
        elif num == ORD_KEY_LEFT:
            x = 10
            y = 3
        elif num == ORD_KEY_RIGHT:
            x = 11
            y = 3
        elif num == ORD_KEY_ESC:
            x = 12
            y = 3
        elif num == ORD_KEY_SPCA:
            x = 8
            y = 3
        elif num == ORD_KEY_SPCB:
            x = 9
            y = 3
        elif num == ORD_MINUS:
            x = 3
            y = 3
        else:
            x = 12
            y = 2
        return x, y
        
        